import json
import os
from typing import Dict, Any

import bottle
from logger import Logger
from servicestub import ServiceStub
from singleton import Singleton


class ArtixBonus(metaclass=Singleton):
    log: Logger
    cases: Dict[str, Dict[str, Any]]

    def __init__(self):
        self.cases = {}
        self.log = Logger()
        self.path = os.path.dirname(os.path.abspath(__file__))
    
    def registerRoutes(self, servicestub: ServiceStub):
        self.log.debug('Регистрация сервиса ArtixBonus')
        self.__updateCases()
        servicestub.route('/ACC/rest/v1/cards/<cardNumber>', method='GET', callback=self.cardByNumber)
        servicestub.route('/ACC/rest/v1/assets/back<transactionData:re:.*>', method='POST', callback=self.transactionBack)
        servicestub.route('/ACC/rest/v1/assets<transactionData:re:.*>', method='POST', callback=self.transaction)
        servicestub.route('/ACC/rest/v1/assets/<sessionId:re:(_cash.*?)>/commit<transactionData:re:(.*?$)>', method='PUT', callback=self.transactionCommit)
        servicestub.route('/ACC/rest/v1/assets/<sessionId:re:(_cash.*?)>/rollback<transactionData:re:(.*?$)>', method='PUT', callback=self.transactionRollback)

    def cardByNumber(self, cardNumber:str = ''):
        self.log.info("Запрос данных о карте по номеру карты: '%s'" % cardNumber)
        self.__updateCases()
        methodPath = '/v1/cards/' + cardNumber

        if methodPath not in self.cases:
            methodPath = '/v1/cards/<cardNumber>'

        return self.__getResult(methodPath)

    def transaction(self, transactionData):
        self.log.info("Провести операцию: данные транзакции: '%s'" % transactionData)
        methodPath = '/v1/assets' + transactionData

        if methodPath not in self.cases:
            methodPath = '/v1/assets<transactionData>'

        return self.__getResult(methodPath)
  
    def transactionBack(self, transactionData):
        self.log.info("Провести операцию отмены: данные транзакции: '%s'" % transactionData)
        methodPath = '/v1/assets/back' + transactionData

        if methodPath not in self.cases:
            methodPath = '/v1/assets/back<transactionData>'

        return self.__getResult(methodPath)
  
    def transactionRollback(self, sessionId='', transactionData=''):
        self.log.info("Закрыть операцию отменой ее воздействия.: '%s', данные транзакции: '%s'" % (sessionId, transactionData))
        methodPath = '/v1/assets/' + sessionId + '/rollback/' + transactionData

        if methodPath not in self.cases:
            methodPath = '/v1/assets/<sessionId>rollback/<transactionData>'
    
        return self.__getResult(methodPath)

    def transactionCommit(self, sessionId='', transactionData=''):
        self.log.info("Закрыть операцию потдверждением ее воздействия..: '%s'" % transactionData)
        methodPath = '/v1/assets/' + sessionId + '/rollback/' + transactionData
 
        if methodPath not in self.cases:
            methodPath = '/v1/assets/<sessionId>commit/<transactionData>'
    
        return self.__getResult(methodPath)


    def __getResult(self, methodPath: str):
          if methodPath not in self.cases:
              raise bottle.HTTPError(400, 'Путь "%s" не существует' % methodPath)

          if not len(self.cases[methodPath]):
              self.log.warn('В кейсах не осталось значений')
              return None

          body = json.dumps(self.cases[methodPath][0].get('response'))
          status = self.cases[methodPath][0]['status']
          response = bottle.HTTPResponse(status=status if status else 200, body=body)
          response.add_header("Content-Type", "application/json")

          return response

    def __updateCases(self):
        try:
              with open(os.path.join(self.path, 'cases.json'), 'r') as cases:
                  self.cases = json.load(cases)
        except Exception as e:
            self.log.error(str(e))


instance = ArtixBonus()
