from typing import Dict, Iterable, Any
import time, json, os

import bottle

from logger import Logger
from servicestub import ServiceStub
from singleton import Singleton


class Podeli(metaclass=Singleton):
    log: Logger
    cases: Dict[str, Iterable[Dict[str, Any]]]

    def __init__(self):
        self.log = Logger()
        self.cases = {}

    def registerRoutes(self, servicestub: ServiceStub):
        self.log.info('Регистрация сервиса %s' % type(self).__name__)
        self.__updateCases()

        servicestub.route('/create', method='POST', callback=self.create)
        servicestub.route('/create_offline', method='POST', callback=self.create_offline)
        servicestub.route('/<orderId>/info', method='GET', callback=self.info)
        servicestub.route('/<orderId>/commit', method='POST', callback=self.commit)
        servicestub.route('/<orderId>/cancel', method='POST', callback=self.cancel)
        servicestub.route('/<orderId>/refund', method='POST', callback=self.refund)

    def create(self):
        self.log.info("Выбор способа оплаты")
        methodPath = '/create'

        return self.__getResult(methodPath)

    def create_offline(self):
        self.log.info("Создание заказа оффлайн")
        methodPath = '/create_offline'

        return self.__getResult(methodPath)

    def info(self, orderId=''):
        self.log.info('Получение информации по заказу %s' % orderId)
        methodPath = '/%s/info/' % orderId
        if methodPath not in self.cases:
            methodPath = '/<orderId>/info'

        return self.__getResult(methodPath)

    def commit(self, orderId=''):
        self.log.info('Подтверждение заказа %s' % orderId)
        methodPath = '/%s/commit/' % orderId
        if methodPath not in self.cases:
            methodPath = '/<orderId>/commit'

        return self.__getResult(methodPath)

    def cancel(self, orderId=''):
        self.log.info('Отмены заказа %s' % orderId)
        methodPath = '/%s/cancel/' % orderId
        if methodPath not in self.cases:
            methodPath = '/<orderId>/cancel'

        return self.__getResult(methodPath)

    def refund(self, orderId=''):
        self.log.info("Возврат товара по заказу '%s'" % orderId)
        methodPath = '/%s/refund/' % orderId
        if methodPath not in self.cases:
            methodPath = '/<orderId>/refund'

        return self.__getResult(methodPath)

    def __getResult(self, methodPath: str):
        if methodPath not in self.cases:
            self.log.warn('Путь "%s" не существует' % methodPath)
            raise bottle.HTTPError(400, 'Путь "%s" не существует' % methodPath)

        if not len(self.cases[methodPath]):
            self.log.warn('В кейсах не осталось значений')
            return None

        time.sleep(self.cases[methodPath][0].get('timeout', 0))

        body = json.dumps(self.cases[methodPath][0].get('response'))
        response = bottle.HTTPResponse(status=self.cases[methodPath][0].get('status'), body=body)
        response.add_header("Content-Type", "application/json")

        headers = self.cases[methodPath][0].get('headers')
        if headers is not None:
            for header in headers.keys():
                response.add_header(header, headers[header])

        # если в кейсе нет счётчика, то возвращаем str(response)
        if 'count' not in self.cases[methodPath][0]:
            return response

        self.cases[methodPath][0]['count'] -= 1

        if self.cases[methodPath][0]['count'] == 0:
            del self.cases[methodPath][0]

        return response

    def __updateCases(self):
        try:
            with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'podeli.json'), 'r') as cases:
                self.cases = json.load(cases)
        except Exception as e:
            self.log.error(str(e))


instance = Podeli()
