from typing import Dict, Any
import time, json, os

import bottle

from logger import Logger
from servicestub import ServiceStub
from singleton import Singleton


class RaiffeisenSbp(metaclass=Singleton):
    log: Logger
    cases: Dict[str, Dict[str, Any]]

    def __init__(self):
        self.log = Logger()
        self.cases = {}

    def registerRoutes(self, servicestub: ServiceStub):
        self.log.info('Регистрация сервиса RaiffeisenSbp')
        self.__updateCases()

        servicestub.route('/payment/v1/orders', method='POST', callback=self.orders)
        servicestub.route('/payment/v1/orders/<orderId>', method='GET', callback=self.ordersStatus)
        servicestub.route('/payments/v1/orders/<orderId>/refunds/<refundId>', method='POST', callback=self.refunds)
        servicestub.route('/payments/v1/orders/<orderId>/refunds/<refundId>', method='GET', callback=self.refundsStatus)
        servicestub.route('/sbp/v2/qrs/<path:re:.*>', method='GET', callback=self.getQrRegistrationData)
        servicestub.route('/sbp/v2/qrs/<refundId:re:.*>', method='DELETE', callback=self.cancelQR)
        servicestub.route('/sbp/v2/qrs', method='POST', callback=self.qrRegistration)
        servicestub.route('/sbp/v1/qr<path:re:.*>', method='GET', callback=self.getQrData)
        servicestub.route('/sbp/v1/refund', method='POST', callback=self.refund)
        servicestub.route('/sbp/v1/refund/<refundId:re:.*>', method='GET', callback=self.getRefundStatus)

    def qrRegistration(self):
        self.log.info("Регистрация QR")
        self.__updateCases()
        return self.__getResult('/sbp/v2/qrs')

    def getQrData(self, path):
        self.log.info("Получение данных по зарегистрированному QR")
        self.__updateCases()
        return self.__getResult('/sbp/v1/qr/')

    def getQrRegistrationData(self, path):
        self.log.info("Получение данных по зарегистрированному QR")
        self.__updateCases()
        return self.__getResult('/sbp/v2/qrs/{qrId}')

    def refund(self):
        self.log.info("Оформление возврата по платежу")
        self.__updateCases()
        return self.__getResult('/sbp/v1/refund')

    def getRefundStatus(self, refundId):
        self.log.info("Получение информации по возврату")
        self.__updateCases()
        return self.__getResult('/sbp/v1/refund<refundId>')

    def cancelQR(self, refundId):
        self.log.info("Отмена QR")
        self.__updateCases()
        return self.__getResult('/sbp/v2/qrs/cancel<refundId>')

    def orders(self):
        self.log.info("Регистрация заказа")
        self.__updateCases()
        return self.__getResult('/payment/v1/orders')

    def ordersStatus(self, orderId=''):
        self.log.info("Получение статуса заказа")
        self.__updateCases()
        return self.__getResult('/payment/v1/orders/statuses')

    def refunds(self, orderId='', refundId=''):
        self.log.info("Отмена заказа")
        self.__updateCases()
        return self.__getResult('/payment/v1/orders/refunds')

    def refundsStatus(self, orderId='', refundId=''):
        self.log.info("Получение статуса отмены")
        self.__updateCases()
        return self.__getResult('/payment/v1/orders/refundsStatuses')

    def __getResult(self, methodPath: str):
        if methodPath not in self.cases:
            raise bottle.HTTPError(400, 'Путь "%s" не существует' % methodPath)

        time.sleep(self.cases[methodPath].get('timeout', 0))

        responseSuffix = self.cases[methodPath].get('responseSuffix', '')

        body = json.dumps(self.cases[methodPath].get('response' + responseSuffix))
        response = bottle.HTTPResponse(status=200, body=body)
        response.add_header("Content-Type", "application/json")

        return response

    def __updateCases(self):
        self.cases.clear()
        try:
            for file in ["raiffeisensbp"]:
                with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), file + '.json'), 'r') as cases:
                    self.cases.update(json.load(cases))
        except Exception as e:
            self.log.error(str(e))


instance = RaiffeisenSbp()
